@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -127,68 +127,64 @@
 #endif
 
 /* A few #defines to reduce SDL footprint.
-   Only effective when library is statically linked.
-   You have to manually edit this file. */
-#ifndef SDL_LEAN_AND_MEAN
-#define SDL_LEAN_AND_MEAN 0
-#endif
+   Only effective when library is statically linked. */
 
 /* Optimized functions from 'SDL_blit_0.c'
    - blit with source bits_per_pixel < 8, palette */
-#ifndef SDL_HAVE_BLIT_0
-#define SDL_HAVE_BLIT_0 !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_BLIT_0) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_BLIT_0 1
 #endif
 
 /* Optimized functions from 'SDL_blit_1.c'
    - blit with source bytes_per_pixel == 1, palette */
-#ifndef SDL_HAVE_BLIT_1
-#define SDL_HAVE_BLIT_1 !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_BLIT_1) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_BLIT_1 1
 #endif
 
 /* Optimized functions from 'SDL_blit_A.c'
    - blit with 'SDL_BLENDMODE_BLEND' blending mode */
-#ifndef SDL_HAVE_BLIT_A
-#define SDL_HAVE_BLIT_A !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_BLIT_A) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_BLIT_A 1
 #endif
 
 /* Optimized functions from 'SDL_blit_N.c'
    - blit with COLORKEY mode, or nothing */
-#ifndef SDL_HAVE_BLIT_N
-#define SDL_HAVE_BLIT_N !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_BLIT_N) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_BLIT_N 1
 #endif
 
 /* Optimized functions from 'SDL_blit_N.c'
    - RGB565 conversion with Lookup tables */
-#ifndef SDL_HAVE_BLIT_N_RGB565
-#define SDL_HAVE_BLIT_N_RGB565 !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_BLIT_N_RGB565) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_BLIT_N_RGB565 1
 #endif
 
 /* Optimized functions from 'SDL_blit_AUTO.c'
    - blit with modulate color, modulate alpha, any blending mode
    - scaling or not */
-#ifndef SDL_HAVE_BLIT_AUTO
-#define SDL_HAVE_BLIT_AUTO !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_BLIT_AUTO) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_BLIT_AUTO 1
 #endif
 
 /* Run-Length-Encoding
    - SDL_SetSurfaceColorKey() called with SDL_RLEACCEL flag */
-#ifndef SDL_HAVE_RLE
-#define SDL_HAVE_RLE !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_RLE) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_RLE 1
 #endif
 
 /* Software SDL_Renderer
    - creation of software renderer
    - *not* general blitting functions
    - {blend,draw}{fillrect,line,point} internal functions */
-#ifndef SDL_VIDEO_RENDER_SW
-#define SDL_VIDEO_RENDER_SW !SDL_LEAN_AND_MEAN
+#if !defined(SDL_VIDEO_RENDER_SW) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_VIDEO_RENDER_SW 1
 #endif
 
 /* YUV formats
    - handling of YUV surfaces
    - blitting and conversion functions */
-#ifndef SDL_HAVE_YUV
-#define SDL_HAVE_YUV !SDL_LEAN_AND_MEAN
+#if !defined(SDL_HAVE_YUV) && !defined(SDL_LEAN_AND_MEAN)
+#define SDL_HAVE_YUV 1
 #endif
 
 #ifdef SDL_RENDER_DISABLED
@@ -207,7 +203,6 @@
 #endif // SDL_RENDER_DISABLED
 
 #ifdef SDL_GPU_DISABLED
-#undef SDL_GPU_D3D11
 #undef SDL_GPU_D3D12
 #undef SDL_GPU_METAL
 #undef SDL_GPU_VULKAN
